@@ -1,7 +1,9 @@
-import { LegendOptions } from '@grafana/ui';
-import { YAxis } from '@grafana/data';
+import { LegendOptions, GraphTooltipOptions } from '@grafana/ui';
+import { YAxis, FieldDisplayOptions } from '@grafana/data';
 
 import { GraphLegendEditorLegendOptions } from './GraphLegendEditor';
+// TODO move out from single stat
+import { standardFieldDisplayOptions } from '../singlestat2/types';
 
 export interface SeriesOptions {
   color?: string;
@@ -20,6 +22,8 @@ export interface Options {
   series: {
     [alias: string]: SeriesOptions;
   };
+  fieldOptions: FieldDisplayOptions;
+  tooltipOptions: GraphTooltipOptions;
 }
 
 export const defaults: Options = {
@@ -34,4 +38,6 @@ export const defaults: Options = {
     placement: 'under',
   },
   series: {},
+  fieldOptions: { ...standardFieldDisplayOptions },
+  tooltipOptions: { mode: 'single' },
 };
