@@ -2,7 +2,9 @@ import _ from 'lodash';
 import { PanelCtrl } from '../../../features/panel/panel_ctrl';
 import { auto } from 'angular';
 import { BackendSrv } from '@grafana/runtime';
+import { PanelEvents } from '@grafana/data';
 import { ContextSrv } from '../../../core/services/context_srv';
+import { CoreEvents } from 'app/types';
 
 class PluginListCtrl extends PanelCtrl {
   static templateUrl = 'module.html';
@@ -22,7 +24,7 @@ class PluginListCtrl extends PanelCtrl {
     _.defaults(this.panel, this.panelDefaults);
 
     this.isAdmin = contextSrv.hasRole('Admin');
-    this.events.on('init-edit-mode', this.onInitEditMode.bind(this));
+    this.events.on(PanelEvents.editModeInitialized, this.onInitEditMode.bind(this));
     this.pluginList = [];
     this.viewModel = [
       { header: 'Installed Apps', list: [], type: 'app' },
@@ -51,7 +53,7 @@ class PluginListCtrl extends PanelCtrl {
     const modalScope = this.$scope.$new(true);
     modalScope.plugin = plugin;
 
-    this.publishAppEvent('show-modal', {
+    this.publishAppEvent(CoreEvents.showModal, {
       src: 'public/app/features/plugins/partials/update_instructions.html',
       scope: modalScope,
     });
